from . import Base

class nekonime(Base):
    host = "https://nekonime.stream"
    desc = "Nonton stream anime"

    def extract(self, id, rextract=False):
        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (eps := soup.find(class_="daftarepi")):
             d = {}
             for a in eps.findAll("a"):
                 d[a.text] = self.getPath(a.attrs["href"])
             key = self.choice(d.keys())
             return self.extract(d[key], rextract=True)

        elif not rextract and (eps := soup.find(class_="flircontainer")):
             for a in eps.findAll("a"):
                 if "All Eps" == a.text:
                     ch = self.choice(["Lanjut", a.text])
                     if ch != "Lanjut":
                         return self.extract(self.getPath(a.attrs["href"]))
        if (ddl := soup.find(class_="soraddl")):
            d = {}
            for p in ddl.findAll(class_="soraurl"):
                result = {}
                for a in p.findAll("a"):
                    result[a.text] = a.attrs["href"]
                d[p.strong.text] = result
            return d


    def search(self, query, page=1):
        raw = self.session.get(f"{self.host}/page/{page}", params={"s": query})
        soup = self.soup(raw)

        for article in soup.findAll(class_="article-body"):
            a = article.find("a")
            yield {
                "title": a.text,
                "id": self.getPath(a.attrs["href"])
            }



from . import Base


class kusonime(Base):
    desc = "Anime Batch"
    host = "https://kusonime.com"

    def extract(self, id):
        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        d = {}
        for n, smokeddl in enumerate(soup.findAll("div", class_="smokeddl")[:-1], start=1):
            title = f"{n}. " + smokeddl.find(class_="smokettl").text
            d[title] = {}
            for smokeurl in smokeddl.findAll(class_="smokeurl"):
                res = smokeurl.strong.text
                links = {}
                for a in smokeurl.findAll("a"):
                    links[a.text] = a.attrs["href"]
                d[title][res] = links

        key = self.choice(d.keys())
        return d[key]

    def search(self, query, page=1):
        raw = self.session.get(self.host + f"/page/{page}/", params={
            "s": query})
        soup = self.soup(raw)

        for item in soup.findAll("div", class_="content"):
            d = {
                "title": item.h2.text,
                "id": self.getPath(item.a.attrs["href"])
            }
            for p in item.findAll("p"):
                if "genre" in p.text:
                    d["genre"] = [a.text for a in p.findAll(p)]
                elif "release" in p.text:
                    d["released"] = p.text.split("on ")[-1]
            yield d



from . import Base


class otakudesu(Base):
    desc = "Anime Batch, OnGoing"
    host = "https://otakudesu.tv"

    def extract(self, id):
        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (eps := soup.findAll(class_="episodelist")[1:]):
            ch = {}
            for li in eps[0].findAll("li"):
                ch[li.a.text] = li.a.attrs["href"]
            id = self.choice(ch.keys())
            return self.extract(self.getPath(ch[id]))
        else:
            dls = {}
            for dl in soup.findAll("div", class_="download"):
                title = [h4.text for h4 in dl.findAll("h4")]
                for n, ul in enumerate(dl.findAll("ul")):
                    if len(title) == 1:
                        n = 0
                    n = title[n]
                    if not dls.get(n):
                        dls[n] = {}
                    for li in ul.findAll("li"):
                        dls[n][f"{li.strong.text}/{li.i.text}"] = {
                            a.text: a.attrs["href"] for a in li.findAll("a")
                        }
            key = self.choice(dls.keys())
            return dls[key]

    def search(self, query, page=1):
        raw = self.session.get(self.host, params={
            "s": query, "post_type": "Anime"})
        soup = self.soup(raw)

        ul = soup.find("ul", class_="chivsrc")
        for li in ul.findAll("li"):
            result = {
                "title": li.a.text,
                "id": self.getPath(li.a.attrs["title"])
            }
            yield result



from . import Base


class oploverz(Base):
    host = "https://www.oploverz.in"
    desc = "Anime, OnGoing"

    def extract(self, id, rextract=False):
        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (ep_list := soup.find(class_="episodelist")):
            d = {}
            for li in ep_list.findAll("li"):
                a = li.find("a")
                d[a.text.strip()] = a.attrs["href"]
            key = self.choice(d.keys())
            return self.extract(d[key], rextract=True)

        elif not rextract and (all_eps := soup.find(class_="btn-full-eps")):
            ch = self.choice(["Lanjut", all_eps.text])
            if ch != "Lanjut":
                return self.extract(self.getPath(all_eps.attrs["href"]))

        if (ddl := soup.find(class_="soraddl")):
            d = {}
            for p in ddl.findAll("p"):
                if (ttl := p.find(class_="sorattl")):
                    result = {}
                    for a in p.findAll("a"):
                        result[a.text] = a.attrs["href"]
                    d[ttl.text] = result
            return d

    def search(self, query, page=1):
        raw = self.session.get(f"{self.host}/page/{page}/",
                               params={"s": query, "post_type": "post"})
        soup = self.soup(raw)

        for dtl in soup.findAll("div", class_="dtl"):
            a = dtl.find("a")
            yield {
                "title": a.text,
                "id": self.getPath(a.attrs["href"]),
                "release": dtl.findAll("span")[1].text
            }



from . import Base


class wibudesu(Base):
    host = "https://wibudesu.com"
    desc = "anime batch"

    def extract(self, id):
    	raw = self.session.get(f"{self.host}/{id}")
    	soup = self.soup(raw)

    	lexot = soup.find(class_="lexot")

    	result = {}
    	for p in lexot.findAll("p")[1:]:
    		if (links := p.findAll("a")):
    			title = p.strong.text
    			d = {}
    			for a in links:
    				d[a.text] = a.attrs["href"]
    			result[title] = d
    	return result

    def search(self, query, page=1):
        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query, "post_type": "post"})
        soup = self.soup(raw)

        for detpost in soup.findAll(class_="detpost"):
            a = detpost.find("a")
            result = {
                "title": a.attrs["title"],
                "id": self.getPath(a.attrs["href"])
            }

            if (morec := detpost.find(class_="morec")):
                result["genre"] = [a.text for a in morec.findAll("a")]

            yield result



import requests
import bs4
import re
import PyInquirer
import sys
from urllib.parse import urlparse

class Base(object):
    def __init__(self):
        self.session = self._build_session()
        self.directRules = {
            "_fembed": re.compile(r"https://layarkacaxxi.icu/f/[^>]+"),
            "_zippyshare": re.compile(r"https://www\d+.zippyshare.com/v/[^/]+/file.html")
        }
        self.re = re
        self.logging = None
        self.counter = 0

    def _build_session(self) -> requests.Session:
        session = requests.Session()
        session.headers[
            "User-Agent"] = "Mozilla/5.0 (Linux; Android 7.0; 5060 Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/58.0.3029.83 Mobile Safari/537.36"
        return session

    def soup(self, raw):
        text = raw.text if hasattr(raw, "text") else raw
        return bs4.BeautifulSoup(text, "html.parser")

    def extract_direct_url(self, url):
        for title, pattern in self.directRules.items():
            if pattern.match(url):
                self.logging.info(f"Mengekstrak {title[1:]}: {url}")
                func = getattr(self, title)
                result = func(url)

                key = self.choice(result.keys())
                return result[key]
        return url

    def getPath(self, url):
        return urlparse(url).path.strip("/")

    def _write(self, raw, file="x.html"):
        if hasattr(raw, "prettify"):
            raw = raw.prettify()
        with open(file, "w") as f:
            f.write(raw)

    def _reformat(self, raw):
        if not isinstance(raw, str) or re.match(r"^\d+\.", raw):
            return raw
        else:
            self.counter += 1
            raw = re.sub(r"^\s+|\s+$", "", raw)
            if self.directRules.get("_" + raw.lower()):
                raw += " (direct)"
            return f"{self.counter}) {raw}"

    def choice(self, choices, msg=None, reset_counter=True):
        if reset_counter:
            self.counter = 0
        choices = list(choices)
        if len(choices) == 0:
            sys.exit("pilihan kosong")

        if len(choices) == 1:
            return choices[0]

        nch = {
            self._reformat(v): v for v in choices}
        questions = [
            {
                "type": "list",
                "name": "js",
                "message": msg or "pilih:",
                "choices": list(nch.keys())
            }
        ]

        output = PyInquirer.prompt(questions)["js"]
        return re.sub(r" \(direct\)$", "", nch[output])

    """
    Extractor
    """

    def _zippyshare(self, url):
        raw = self.session.get(url)
        res = re.search(
            r'href = "(?P<i>[^"]+)" \+ \((?P<t>[^>]+?)\) \+ "(?P<f>[^"]+)', raw.text)
        if res is not None:
            res = res.groupdict()
            return {
                "": re.search(r"(^https://www\d+.zippyshare.com)", raw.url).group(1) +
                res["i"] + str(eval(res["t"])) + res["f"]}
        return {"": url}

    def _fembed(self, url):
        raw = self.session.get(url)
        api = re.search(r"(/api/source/[^\"']+)", raw.text)
        if api is not None:
            result = {}
            raw = self.session.post(
                "https://layarkacaxxi.icu" + api.group(1)).json()
            for d in raw["data"]:
                f = d["file"]
                direct = requests.head(f).headers.get("Location", f)
                result[f"{d['label']}/{d['type']}"] = direct
            return result



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['lk21', 'lk21.extractors']

package_data = \
{'': ['*']}

install_requires = \
['PyInquirer', 'bs4', 'requests', 'colorama']

entry_points = \
{'console_scripts': ['lk21 = lk21:main']}

setup(name='lk21',
      version='1.1.91',
      description='cari film dan anime lewat terminal',
      author='Val',
      author_email='apklover76@gmail.com',
      url='https://github.com/zevtyardt',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
     )



from . import Base


class lk21(Base):
    desc = "Film, Movie (default)"
    host = "http://149.56.24.226/"

    def extract(self, id: str):
        raw = self.session.post("http://dl.sharemydrive.xyz/verifying.php",
                                headers={
                                    "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
                                    "Accept": "*/*",
                                    "X-Requested-With": "XMLHttpRequest"
                                },
                                params={"slug": id},
                                data={"slug": id}
                                )
        soup = self.soup(raw)
        tb = soup.find("tbody")

        result = {}
        for tr in tb.findAll("tr"):
            title = tr.find("strong").text
            result[title] = {}
            for td in tr.findAll("td")[1:]:
                if (a := td.a):
                    result[title][a.attrs["class"]
                                  [-1].split("-")[-1]] = a.attrs["href"]
        return result

    def search(self, title: str, page=1):
        if page > 1:
            return
        raw = self.session.get(self.host,
                               params={"s": title})

        soup = self.soup(raw)
        for item in soup.findAll(class_="search-item"):
            a = item.a
            extra = {"genre": [], "star": [], "country": [],
                     "size": [""], "quality": [""], "year": [""]}
            for tag in item.find(class_="cat-links").findAll("a"):
                name, it = self.re.findall(r"/([^/]+)/([^/]+)", str(tag))[0]
                extra[name].insert(0, it)

            for p in filter(lambda x: x.strong is not None, item.findAll("p")):
                np, vl = self.re.findall(
                    r"^([^:]+):\s+(.+)", p.text.strip())[0]
                np = "star" if np == "Bintang" else "director" if np == "Sutradara" else np
                extra[np] = self.re.split(r"\s*,\s*", vl) if "," in vl else vl

            extra["id"] = self.re.search(
                r"\w/([^/]+)", a.attrs["href"]).group(1)
            result = {
                "title": (item.find("h2").text or a.img.attrs["alt"]).strip(),
            }
            result.update(extra)
            yield result



#!/usr/bin/python
import colorama
colorama.init()

from .extractors import Base
import json
import re
import argparse
import json
import sys
import PyInquirer

import logging
import importlib
import glob
import os

from shutil import get_terminal_size
from urllib.parse import urlparse

basedir = os.path.join(os.path.dirname(__file__), "extractors")
for file in glob.glob(f"{basedir}/*.py"):
    filename = os.path.basename(file)
    if not filename.startswith("__"):
        importlib.import_module(f"lk21.extractors.{filename[:-3]}")

logging.basicConfig(format="\x1b[K%(message)s", level=logging.INFO)
logging.getLogger("requests").setLevel(logging.WARNING)

def parse_range(raw):
    assert re.match(r"[\s\d:]+", raw), f"invalid syntax: {raw!r}"
    if "," in raw:
        for rg in re.split(r"\s*,\s*", raw):
            if ":" not in rg:
                yield rg
            else:
                assert len(re.findall(r":", raw)
                           ) <= 1, f"invalid syntax: {raw!r}"
                yield from parse_range(rg)
    else:
        spl = re.split(r"\s*:\s*", raw)
        if not spl[0]:
            spl[0] = "1"

        start, end = map(lambda x: int(x) if x else None, spl)
        if end:
            assert start < end, "angka pertama tidak boleh lebih dari angka kedua"
            yield from range(start, end + 1)
        else:
            while True:
                yield start
                start += 1


def title(text, rtn=False):
    r = f" [\x1b[92m{text}\x1b[0m]"
    if rtn:
        return r
    logging.info(r)


def main():
    global extractor

    extractors = {
        obj.__name__: obj for obj in Base.__subclasses__() if obj
    }

    parser = argparse.ArgumentParser(
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, max_help_position=get_terminal_size().lines))
    parser.add_argument("query", metavar="query",
                        nargs="*", help="kueri, judul")
    parser.add_argument("-p", metavar="page", dest="page",
                        help=("halaman situs, contoh penggunaan:\n"
                              "  - 1,2,3\n"
                              "  - 1:2, 2:8\n"
                              "  - 1,2,3:8\n"
                              "  - default halaman pertama\n"
                              "    dan seterusnya"), type=str, default="1:")
    parser.add_argument("-i", "--information", dest="info",
                        action="store_true", help="cetak informasi dari item yang dipilih")
    parser.add_argument("-c", "--copy", metavar="command",
                        help=("salin url ke papan klip\n"
                              "contoh: termux-clipboard-set {}\n\n"
                              "        wajib ada {} karena akan ditimpa\n"
                              "        dengan url yang dipilih"))

    extractor_group = parser.add_argument_group("Daftar Situs",
                                                description=(
                                                    f"pilih salah satu dari ke-{len(extractors)} situs berikut:"
                                                ))
    extractor = extractor_group.add_mutually_exclusive_group()
    for egn, kls in extractors.items():
        netloc = urlparse(kls.host).netloc
        for index in range(1, len(egn)):
            try:
                arg = [f"-{egn[:index]}"]
                if arg[0] != f"-{egn}":
                    arg.append(f"--{egn}")
                extractor.add_argument(*arg, action="store_true",
                                       help=f"{netloc}: {kls.desc.capitalize()}")
                break
            except argparse.ArgumentError:
                continue

    args = parser.parse_args()

    if not args.query or (args.copy and "{}" not in args.copy):
        parser.print_help()
        sys.exit(0)

    extractor = extractors["lk21"]
    for egn, kls in extractors.items():
        if args.__dict__[egn]:
            extractor = kls
            break
    extractor = extractor()
    extractor.logging = logging
    query = " ".join(args.query)

    id = False
    nextPage = True
    Range = parse_range(args.page)
    netloc = urlparse(extractor.host).netloc
    try:
        page = Range.__next__()
        cache = {page: list(extractor.search(query, page=page))}
        while not id:
            print(
                f"Mencari {query!r} -> {netloc} halaman {page}")
            logging.info(
                f"Total item terkumpul: {sum(len(v) for v in cache.values())} item dari {len(cache)} halaman")
            if not cache[page]:
                exit("Not Found")

            if len(cache[page]) == 1:
                response = f"1. " + cache[page][0]["title"]
            else:
                response = extractor.choice([
                    i['title'] for i in cache[page]] + [PyInquirer.Separator(), "00. Back", "01. Next", "02. Exit"], reset_counter=False)
            pgs = list(cache.keys())
            index = pgs.index(page)
            if response.endswith("Exit"):
                break
            elif response.endswith("Back"):
                if extractor.counter > -1:
                    extractor.counter -= len(cache[page])
                print("\x1b[3A\x1b[K", end="")
                if index > 0 and len(pgs) > 1:
                    page = pgs[index - 1]
                    extractor.counter -= len(cache[page])
            elif response.endswith("Next") and nextPage is True:
                if index >= len(pgs) - 1:
                    try:
                        page = Range.__next__()
                        if len(res := list(extractor.search(query, page=page))) > 0:
                            cache[page] = res
                    except StopIteration:
                        nextPage = False
                else:
                    page = pgs[index + 1]
                if nextPage:
                    print("\x1b[3A\x1b[K", end="")
            else:
                for r in cache[page]:
                    if r.get("title") == re.sub(r"^\d+\. ", "", response):
                        if args.info:
                            logging.info(
                                f"\n [\x1b[92m{r.pop('title')}\x1b[0m]")
                            for k, v in r.items():
                                logging.info(
                                    f"   {k}: {', '.join(filter(lambda x: x, v)) if isinstance(v, list) else v}")
                            logging.info("")

                        id = r["id"]
                        break
        if id:
            logging.info(f"Mengekstrak link unduhan: {id}")
            result = extractor.extract(id)
            while any(isinstance(v, dict) for v in result.values()):
                key = extractor.choice(result.keys())
                result = result[key]
            key1 = extractor.choice(result.keys())
            url = extractor.extract_direct_url(result[key1])

            if args.copy:
                logging.info(f"Menyetel papan klip: {url}")
                os.system(args.copy.format(url))
            else:
                logging.info("")
                title("Url Dipilih")
                logging.info(f"\n{url}\n")

    except Exception as e:
        logging.info(f"{e}")
